@@ -12,17 +12,14 @@ from qa.data_loader import get_feature_from_data
 from utility.loss import *
 
 
-class BertQA(nn.Module):
+class QA(nn.Module):
 
-    def __init__(self, model_config, maxlen=512, dropout=0.1):
+    def __init__(self, tokenizer, pretrained, maxlen=128, dropout=0.1):
         super().__init__()
         self.device = 'cuda' if torch.cuda.is_available() else 'cpu'
         print('Using device:', self.device)
-        if 'albert_chinese' in model_config:
-            self.tokenizer = BertTokenizer.from_pretrained(model_config)
-        else:
-            self.tokenizer = AutoTokenizer.from_pretrained(model_config)
-        self.pretrained = AutoModel.from_pretrained(model_config)
+        self.tokenizer = tokenizer
+        self.pretrained = pretrained
         self.maxlen = maxlen
 
         self.dropout = nn.Dropout(dropout)
@@ -35,10 +32,10 @@ class BertQA(nn.Module):
         self.loss_fct = self.loss_fct.to(self.device)
 
     def forward(self, batch_data, eval=False):
-        inputs = torch.tensor(batch_data['input']).to(self.device)
-        masks = torch.tensor(batch_data['mask']).to(self.device)
+        inputs = torch.as_tensor(batch_data['input']).to(self.device)
+        masks = torch.as_tensor(batch_data['mask']).to(self.device)
 
-        targets = torch.tensor(batch_data['target']).to(self.device)
+        targets = torch.as_tensor(batch_data['target']).to(self.device)
         start_positions, end_positions = targets.split(1, dim=1)
         start_positions = start_positions.squeeze(1)
         end_positions = end_positions.squeeze(1)
@@ -54,8 +51,8 @@ class BertQA(nn.Module):
                 'label_prob_all': [],
                 'label_map': []
             }
-            reshaped_start_logits = softmax(start_logits)
-            reshaped_end_logits = softmax(end_logits)
+            reshaped_start_logits = softmax(start_logits,dim=1)
+            reshaped_end_logits = softmax(end_logits,dim=1)
             start_prob = reshaped_start_logits.data.tolist()[0]
             end_prob = reshaped_end_logits.data.tolist()[0]
             result_dict['label_prob_all'].append({'start': dict(zip(range(len(start_prob)), start_prob))})
