@@ -10,7 +10,12 @@ import tensorboardX as tensorboard
 from torch.utils import data
 from itertools import zip_longest
 import os
-import tfkit
+import gen_once
+import gen_twice
+import gen_onebyone
+import classifier
+import tag
+import qa
 
 
 def write_log(*args):
@@ -32,7 +37,7 @@ def optimizer(model, lr):
     return optimizer
 
 
-def train(models_list, train_dataset, arg, fname, epoch, maxlen):
+def train(models_list, train_dataset, models_tag, arg, epoch):
     optims = []
     models = []
     for i, m in enumerate(models_list):
@@ -49,7 +54,7 @@ def train(models_list, train_dataset, arg, fname, epoch, maxlen):
     end = False
     pbar = tqdm(total=total_iter_length)
     while not end:
-        for model, optim, batch in zip(models, optims, iters):
+        for model, optim, mtag, batch in zip(models, optims, models_tag, iters):
             train_batch = next(batch, None)
             if train_batch is not None:
                 loss = model(train_batch)
@@ -61,7 +66,7 @@ def train(models_list, train_dataset, arg, fname, epoch, maxlen):
                     writer.add_scalar("loss/step", loss.mean().item(), epoch)
                 if total_iter % 100 == 0 and total_iter != 0:  # monitoring
                     write_log(
-                        f"model: {model.module.__class__.__name__}, step: {total_iter}, loss: {t_loss / total_iter if total_iter > 0 else 0}, total:{total_iter_length}")
+                        f"tag: {mtag}, model: {model.module.__class__.__name__}, step: {total_iter}, loss: {t_loss / total_iter if total_iter > 0 else 0}, total:{total_iter_length}")
             else:
                 end = True
         pbar.update(1)
@@ -126,6 +131,7 @@ def main():
     parser.add_argument("--model", type=str, required=True, nargs='+',
                         choices=['once', 'twice', 'onebyone', 'classify', 'tagRow', 'tagCol', 'qa',
                                  'onebyone-neg', 'onebyone-pos', 'onebyone-both', ])
+    parser.add_argument("--tag", type=str, nargs='+', help="tag to identity task in multi-task")
     parser.add_argument("--config", type=str, default='bert-base-multilingual-cased', required=True,
                         help='distilbert-base-multilingual-cased/bert-base-multilingual-cased/voidful/albert_chinese_small')
     parser.add_argument("--seed", type=int, default=609)
@@ -141,7 +147,7 @@ def main():
 
     write_log("TRAIN PARAMETER")
     write_log("=======================")
-    write_log(vars(arg))
+    [write_log(var, ':', vars(arg)[var]) for var in vars(arg)]
     write_log("=======================")
 
     # load pre-train model
@@ -152,6 +158,7 @@ def main():
     pretrained = AutoModel.from_pretrained(arg.config)
 
     models = []
+    models_tag = arg.tag if arg.tag is not None else [m.lower() + "_" + str(ind) for ind, m in enumerate(arg.model)]
     train_dataset = []
     test_dataset = []
     train_ds_maxlen = 0
@@ -159,44 +166,44 @@ def main():
     for model_type, train_file, valid_file in zip_longest(arg.model, arg.train, arg.valid, fillvalue=""):
         model_type = model_type.lower()
         if "once" in model_type:
-            train_ds = tfkit.gen_once.loadOnceDataset(train_file, pretrained=arg.config, maxlen=arg.maxlen,
-                                                      cache=arg.cache)
-            test_ds = tfkit.gen_once.loadOnceDataset(valid_file, pretrained=arg.config, maxlen=arg.maxlen,
-                                                     cache=arg.cache)
-            model = tfkit.gen_once.Once(tokenizer, pretrained, maxlen=arg.maxlen)
+            train_ds = gen_once.loadOnceDataset(train_file, pretrained=arg.config, maxlen=arg.maxlen,
+                                                cache=arg.cache)
+            test_ds = gen_once.loadOnceDataset(valid_file, pretrained=arg.config, maxlen=arg.maxlen,
+                                               cache=arg.cache)
+            model = gen_once.Once(tokenizer, pretrained, maxlen=arg.maxlen)
         elif "twice" in model_type:
-            train_ds = tfkit.gen_once.loadOnceDataset(train_file, pretrained=arg.config, maxlen=arg.maxlen,
-                                                      cache=arg.cache)
-            test_ds = tfkit.gen_once.loadOnceDataset(valid_file, pretrained=arg.config, maxlen=arg.maxlen,
-                                                     cache=arg.cache)
-            model = tfkit.gen_twice.Twice(tokenizer, pretrained, maxlen=arg.maxlen)
+            train_ds = gen_once.loadOnceDataset(train_file, pretrained=arg.config, maxlen=arg.maxlen,
+                                                cache=arg.cache)
+            test_ds = gen_once.loadOnceDataset(valid_file, pretrained=arg.config, maxlen=arg.maxlen,
+                                               cache=arg.cache)
+            model = gen_twice.Twice(tokenizer, pretrained, maxlen=arg.maxlen)
         elif "onebyone" in model_type:
-            train_ds = tfkit.gen_onebyone.loadOneByOneDataset(train_file, pretrained=arg.config, maxlen=arg.maxlen,
-                                                              cache=arg.cache,
-                                                              likelihood=model_type)
-            test_ds = tfkit.gen_onebyone.loadOneByOneDataset(valid_file, pretrained=arg.config, maxlen=arg.maxlen,
-                                                             cache=arg.cache)
-            model = tfkit.gen_onebyone.OneByOne(tokenizer, pretrained, maxlen=arg.maxlen)
+            train_ds = gen_onebyone.loadOneByOneDataset(train_file, pretrained=arg.config, maxlen=arg.maxlen,
+                                                        cache=arg.cache,
+                                                        likelihood=model_type)
+            test_ds = gen_onebyone.loadOneByOneDataset(valid_file, pretrained=arg.config, maxlen=arg.maxlen,
+                                                       cache=arg.cache)
+            model = gen_onebyone.OneByOne(tokenizer, pretrained, maxlen=arg.maxlen)
         elif 'classify' in model_type:
-            train_ds = tfkit.classifier.loadClassifierDataset(train_file, pretrained=arg.config, cache=arg.cache)
-            test_ds = tfkit.classifier.loadClassifierDataset(valid_file, pretrained=arg.config, cache=arg.cache)
-            model = tfkit.classifier.MtClassifier(train_ds.task, tokenizer, pretrained)
+            train_ds = classifier.loadClassifierDataset(train_file, pretrained=arg.config, cache=arg.cache)
+            test_ds = classifier.loadClassifierDataset(valid_file, pretrained=arg.config, cache=arg.cache)
+            model = classifier.MtClassifier(train_ds.task, tokenizer, pretrained)
         elif 'tag' in model_type:
             if "row" in model_type:
-                train_ds = tfkit.tag.loadRowTaggerDataset(train_file, pretrained=arg.config, maxlen=arg.maxlen,
-                                                          cache=arg.cache)
-                test_ds = tfkit.tag.loadRowTaggerDataset(valid_file, pretrained=arg.config, maxlen=arg.maxlen,
-                                                         cache=arg.cache)
+                train_ds = tag.loadRowTaggerDataset(train_file, pretrained=arg.config, maxlen=arg.maxlen,
+                                                    cache=arg.cache)
+                test_ds = tag.loadRowTaggerDataset(valid_file, pretrained=arg.config, maxlen=arg.maxlen,
+                                                   cache=arg.cache)
             elif "col" in model_type:
-                train_ds = tfkit.tag.loadColTaggerDataset(train_file, pretrained=arg.config, maxlen=arg.maxlen,
-                                                          cache=arg.cache)
-                test_ds = tfkit.tag.loadColTaggerDataset(valid_file, pretrained=arg.config, maxlen=arg.maxlen,
-                                                         cache=arg.cache)
-            model = tfkit.tag.Tagger(train_ds.label, tokenizer, pretrained, maxlen=arg.maxlen)
+                train_ds = tag.loadColTaggerDataset(train_file, pretrained=arg.config, maxlen=arg.maxlen,
+                                                    cache=arg.cache)
+                test_ds = tag.loadColTaggerDataset(valid_file, pretrained=arg.config, maxlen=arg.maxlen,
+                                                   cache=arg.cache)
+            model = tag.Tagger(train_ds.label, tokenizer, pretrained, maxlen=arg.maxlen)
         elif 'qa' in model_type:
-            train_ds = tfkit.qa.loadQADataset(train_file, pretrained=arg.config, cache=arg.cache)
-            test_ds = tfkit.qa.loadQADataset(valid_file, pretrained=arg.config, cache=arg.cache)
-            model = tfkit.qa.QA(tokenizer, pretrained, maxlen=arg.maxlen)
+            train_ds = qa.loadQADataset(train_file, pretrained=arg.config, cache=arg.cache)
+            test_ds = qa.loadQADataset(valid_file, pretrained=arg.config, cache=arg.cache)
+            model = qa.QA(tokenizer, pretrained, maxlen=arg.maxlen)
 
         model = model.to(device)
         train_ds_maxlen = train_ds.__len__() if train_ds.__len__() > train_ds_maxlen else train_ds_maxlen
@@ -227,13 +234,14 @@ def main():
     start_epoch = 1
 
     if arg.resume:
+        print("Loading back:", arg.resume)
         package = torch.load(arg.resume, map_location=device)
         if 'model_state_dict' in package:
-            model.load_state_dict(package['model_state_dict'])
-            models.append(model)
+            models[0].load_state_dict(package['model_state_dict'])
         else:
-            for state_dict in package['models']:
-                models.append(model.load_state_dict(state_dict))
+            for model_tag, state_dict in zip(package['tags'], package['models']):
+                tag_ind = package['tags'].index(model_tag)
+                models[tag_ind].load_state_dict(state_dict)
         start_epoch = int(package.get('epoch', 1))
 
     set_seed(arg.seed)
@@ -243,12 +251,13 @@ def main():
         fname = os.path.join(arg.savedir, str(epoch))
 
         write_log(f"=========train at epoch={epoch}=========")
-        train_avg_loss = train(models, train_dataset, arg, fname, epoch, train_ds_maxlen)
+        train_avg_loss = train(models, train_dataset, models_tag, arg, epoch)
 
         write_log(f"=========save at epoch={epoch}=========")
         save_model = {
             'models': [m.state_dict() for m in models],
             'model_config': arg.config,
+            'tags': models_tag,
             'type': arg.model,
             'maxlen': arg.maxlen,
             'epoch': epoch
